#ifndef PANEL_CP2K_H
#define PANEL_CP2K_H

#ifndef PANEL_H
#include "Panel.h"
#endif

#ifndef DYNAMIC_SINGLETON_H
#include "Dynamic_singleton.h"
#endif

#ifndef UI_PANEL_CP2K_H
#include "ui_Panel_CP2K.h"
#endif

#include <QProcess>
#include <QFile>

namespace MM
{
class Model;
class Main_window;

class Panel_CP2K 
:
    public Panel,
    public Ui_Panel_CP2K,
    public Dynamic_singleton <Panel_CP2K>
{
    Q_OBJECT

    Model *         model_;
    QProcess *      process_;
    Text            CP2K_dir_;
    QFile           output_;
    Main_window *   player_;

    bool            read_atom_coords_flag_;
    bool            read_charges_flag_;
    bool            read_moments_flag_;
    bool            user_stop_flag_;
    bool            success_;
    int             optimization_it_;

public:    
    Panel_CP2K ();
   ~Panel_CP2K ();

public slots:
    void            show_options (const QString & text);

    void            run   ();
    void            tune  ();
    void            start ();
    void            stop  ();

    void            started ();
    void            finished (int exitCode, QProcess::ExitStatus exitStatus);

    void            read ();

protected slots:

protected:
    Text const &    CP2K_dir () const {return CP2K_dir_;}
    Text const &    setup_CP2K_dir ();

    void            read_atom_coords (char *buf);
    void            read_charges     (char *buf);
    void            read_moments     (char *buf);
    void            create_input     ();

    Text            basis ();
    Text            data  ();

    bool            not_semiempirical ();
    void            set_stop_state ();
    void            set_run_state  ();

    void            turn (bool on);
    void            closeEvent (QCloseEvent * event);
};

}//MM

#endif //PANEL_CP2K_H
