#include "Panel_preset.h"

#include "Path.h"
#include "Model_kit.h"
#include "Project.h"
//#include "Main_window.h"
#include "Qt_interface.h"

namespace MM
{   

Panel_preset::
Panel_preset ()
:
    Panel ("Theme", "left")
{
    setupUi (this);

    pushButton_Current_model->hide();

    toolButton_Liquid->setIcon 
        (QIcon(Path::icon()+"Liquid in periodic box.png"));

    toolButton_Implicit->setIcon 
        (QIcon(Path::icon()+"DNA with implicit water.png"));

    toolButton_Quantum->setIcon 
        (QIcon(Path::icon()+"Quantum calculations.png"));

    connect(toolButton_Liquid,          SIGNAL(toggled (bool)),
            this,                       SLOT(liquid(bool)));
    connect(toolButton_Implicit,        SIGNAL(toggled (bool)),
            this,                       SLOT(implicit(bool)));
    connect(toolButton_Quantum,         SIGNAL(toggled (bool)),
            this,                       SLOT(quantum(bool)));
       
    connect(pushButton_Current_model,   SIGNAL(pressed ()),
            this,                       SLOT(apply_to_current_model()));
    connect(pushButton_All_models,      SIGNAL(pressed ()),
            this,                       SLOT(apply_to_all()));
       
    quantum  (false);
    implicit (false);
    liquid   (true);
}

void Panel_preset::liquid   (bool on) {label_Liquid  ->setEnabled (on);}
void Panel_preset::implicit (bool on) {label_Implicit->setEnabled (on);}
void Panel_preset::quantum  (bool on) {label_Quantum ->setEnabled (on);}

void Panel_preset::
apply_to_current_model ()
{
    apply (model());
}

void Panel_preset::
apply_to_all ()
{
    Project & project = Project::singleton ();

    for (int i=0;  i<project.model_count();  ++i)
        apply (project.model (i));
}

void Panel_preset::
apply (Model & model)
{
    Model_kit & kit = model.kit();
    QAction *action = Qt_interface::action ("&Ad hoc");

    if (toolButton_Liquid->isChecked ())
    {
        action->setChecked (true);
        action->trigger ();

        kit.set_force_field ("AMBER94");
        kit.MD      ().set_desired_temperature (Temperature_unit ());
#ifdef M_DYNAMIX
        kit.MDynaMix().set_desired_temperature (Temperature_unit ());
#endif
    }
    else if (toolButton_Implicit->isChecked ())
    {
        action->setChecked (true);
        action->trigger ();

        kit.set_force_field ("A94-A1");
        kit.MD().      set_desired_temperature (Temperature_unit().set_K (50));
#ifdef M_DYNAMIX
        kit.MDynaMix().set_desired_temperature (Temperature_unit().set_K (50));
#endif
    }
    else if (toolButton_Quantum->isChecked ())
    {
        action->setChecked (false);
        action->trigger ();

        kit.MD().      set_desired_temperature (Temperature_unit().set_K (50));
#ifdef M_DYNAMIX
        kit.MDynaMix().set_desired_temperature (Temperature_unit().set_K (50));
#endif
    }
}

}//MM
